Extension { #name : 'CPointer' }

{ #category : 'Private' }
CPointer class >> basicNew [
 "disallowed"
  self shouldNotImplement: #basicNew

]

{ #category : 'Private' }
CPointer class >> new [
 "disallowed"
  self shouldNotImplement: #new

]

{ #category : 'Instance creation' }
CPointer class >> newFrom: aCByteArray [

"Return a new instance of CPointer which is a reference to body
 of a CByteArray.
 The result can be used as an argument a CFunction which has
 type #'ptr' or #'&ptr' . "

^ self newNull _initFrom: aCByteArray offset: 0

]

{ #category : 'Instance creation' }
CPointer class >> newNull [
  "Create a new instance with value NULL.
   The result can be used as an argument a CFunction which has
   type #'ptr' or #'&ptr' .

   Non-NULL instances are also returned from CFunction calls when result
   type is  #ptr ; pass such instances to CByteArray classmethods to
   get an instance of CByteArray with which to access the data.
 "

<primitive: 674>  "instance is registered with VM for CData finalization"
^ self _primitiveFailed: #newNull

]

{ #category : 'Private' }
CPointer >> _initFrom: aCByteArray offset: zeroBasedOffset [

 <primitive: 718>
 aCByteArray _validateClass: CByteArray .
 aCByteArray memoryAddress == 0 ifTrue:[
   "CByteArray might be a committed instance faulted."
   ArgumentError signal:'CByteArray has NULL C memory address'.
 ].
 zeroBasedOffset _validateClass: SmallInteger .
 ArgumentError signal:'zeroBasedOffset out of bounds of aCByteArray'.
 self _primitiveFailed: #_initFrom:offset:
      args: { aCByteArray . zeroBasedOffset }

]

{ #category : 'Formatting' }
CPointer >> _inspect [
  | res |
  (res := String new)
    add: ' address=0x' ; add: self memoryAddress asHexString .
  ^ res

]

{ #category : 'Formatting' }
CPointer >> asString [
  | res |
  (res := super asString ) add: self _inspect .
  ^ res

]

{ #category : 'Instance creation' }
CPointer >> memoryAddress [
  "Returns the starting address of the C memory, as a signed 64bit Integer."

<primitive: 846>
^ self _primitiveFailed: #memoryAddress

]
